# -*- coding: utf-8 -*
import os

from flyai.model.t import get_token
from flyai.processor.download import download_model

DOMAIN = "https://www.flyai.com"


class Base(object):

    def predict(self, data):
        '''
        使用模型
        :param path: 模型所在的路径
        :param data: 输入的数据
        :param name: 模型的名字
        :param keep: 模型的keep_prod
        :return: 返回预测的类型
        '''
        pass

    def predict_all(self, datas):
        '''
        使用模型
        :param datas: 输入的数据列表
        :return: 返回预测的数据列表
        '''
        pass

    def save_model(self, session, path, name, overwrite=False):
        '''
        保存模型
        :param session: 训练模型的sessopm
        :param path: 要保存模型的路径
        :param name: 要保存模型的名字
        :param overwrite: 是否覆盖当前模型
        :return:
        '''
        self.check(path, overwrite)

    def check(self, path, overwrite=False):
        '''
        检测模型路径是否存在，是否被覆盖
        :param path: 要保存的路径
        :param overwrite: 是否覆盖
        :return:
        '''
        if overwrite:
            for root, dirs, files in os.walk(path, topdown=False):
                for name in files:
                    os.remove(os.path.join(root, name))
                for name in dirs:
                    os.rmdir(os.path.join(root, name))
        if not os.path.exists(path):
            os.makedirs(path)

    def get_remote_data(self, remote_name):
        if "http" in remote_name:
            token = get_token(DOMAIN)
            if token is not None:
                return download_model(remote_name + "?token=" + get_token(),
                                      os.path.join(os.curdir, 'data', 'input', 'model'),
                                      is_print=True)
            else:
                return None
